/****************************************************************************
** Meta object code from reading C++ file 'ExprColorCurve.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.10.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../kseexpr/src/KSeExprUI/ExprColorCurve.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ExprColorCurve.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.10.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11CCurveSceneE_t {};
} // unnamed namespace

template <> constexpr inline auto CCurveScene::qt_create_metaobjectdata<qt_meta_tag_ZN11CCurveSceneE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "CCurveScene",
        "cvSelected",
        "",
        "x",
        "KSeExpr::Vec3d",
        "y",
        "T_INTERP",
        "interp",
        "curveChanged",
        "interpChanged",
        "selPosChanged",
        "pos",
        "selValChanged",
        "val",
        "resize",
        "width",
        "height"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'cvSelected'
        QtMocHelpers::SignalData<void(double, KSeExpr::Vec3d, T_INTERP)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 }, { 0x80000000 | 4, 5 }, { 0x80000000 | 6, 7 },
        }}),
        // Signal 'curveChanged'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'interpChanged'
        QtMocHelpers::SlotData<void(int)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 7 },
        }}),
        // Slot 'selPosChanged'
        QtMocHelpers::SlotData<void(double)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 11 },
        }}),
        // Slot 'selValChanged'
        QtMocHelpers::SlotData<void(const KSeExpr::Vec3d &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 13 },
        }}),
        // Slot 'resize'
        QtMocHelpers::SlotData<void(int, int)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 15 }, { QMetaType::Int, 16 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CCurveScene, qt_meta_tag_ZN11CCurveSceneE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject CCurveScene::staticMetaObject = { {
    QMetaObject::SuperData::link<QGraphicsScene::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11CCurveSceneE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11CCurveSceneE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11CCurveSceneE_t>.metaTypes,
    nullptr
} };

void CCurveScene::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CCurveScene *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->cvSelected((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KSeExpr::Vec3d>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<T_INTERP>>(_a[3]))); break;
        case 1: _t->curveChanged(); break;
        case 2: _t->interpChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->selPosChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 4: _t->selValChanged((*reinterpret_cast<std::add_pointer_t<KSeExpr::Vec3d>>(_a[1]))); break;
        case 5: _t->resize((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (CCurveScene::*)(double , KSeExpr::Vec3d , T_INTERP )>(_a, &CCurveScene::cvSelected, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (CCurveScene::*)()>(_a, &CCurveScene::curveChanged, 1))
            return;
    }
}

const QMetaObject *CCurveScene::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CCurveScene::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11CCurveSceneE_t>.strings))
        return static_cast<void*>(this);
    return QGraphicsScene::qt_metacast(_clname);
}

int CCurveScene::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QGraphicsScene::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void CCurveScene::cvSelected(double _t1, KSeExpr::Vec3d _t2, T_INTERP _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3);
}

// SIGNAL 1
void CCurveScene::curveChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
namespace {
struct qt_meta_tag_ZN14ExprCBoxWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto ExprCBoxWidget::qt_create_metaobjectdata<qt_meta_tag_ZN14ExprCBoxWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ExprCBoxWidget"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ExprCBoxWidget, qt_meta_tag_ZN14ExprCBoxWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ExprCBoxWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14ExprCBoxWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14ExprCBoxWidgetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14ExprCBoxWidgetE_t>.metaTypes,
    nullptr
} };

void ExprCBoxWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ExprCBoxWidget *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *ExprCBoxWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ExprCBoxWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14ExprCBoxWidgetE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int ExprCBoxWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN16ExprCSwatchFrameE_t {};
} // unnamed namespace

template <> constexpr inline auto ExprCSwatchFrame::qt_create_metaobjectdata<qt_meta_tag_ZN16ExprCSwatchFrameE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ExprCSwatchFrame",
        "selValChangedSignal",
        "",
        "KSeExpr::Vec3d",
        "value",
        "swatchChanged",
        "QColor",
        "color"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'selValChangedSignal'
        QtMocHelpers::SignalData<void(KSeExpr::Vec3d)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'swatchChanged'
        QtMocHelpers::SignalData<void(QColor)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ExprCSwatchFrame, qt_meta_tag_ZN16ExprCSwatchFrameE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ExprCSwatchFrame::staticMetaObject = { {
    QMetaObject::SuperData::link<QFrame::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16ExprCSwatchFrameE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16ExprCSwatchFrameE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16ExprCSwatchFrameE_t>.metaTypes,
    nullptr
} };

void ExprCSwatchFrame::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ExprCSwatchFrame *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->selValChangedSignal((*reinterpret_cast<std::add_pointer_t<KSeExpr::Vec3d>>(_a[1]))); break;
        case 1: _t->swatchChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ExprCSwatchFrame::*)(KSeExpr::Vec3d )>(_a, &ExprCSwatchFrame::selValChangedSignal, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ExprCSwatchFrame::*)(QColor )>(_a, &ExprCSwatchFrame::swatchChanged, 1))
            return;
    }
}

const QMetaObject *ExprCSwatchFrame::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ExprCSwatchFrame::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16ExprCSwatchFrameE_t>.strings))
        return static_cast<void*>(this);
    return QFrame::qt_metacast(_clname);
}

int ExprCSwatchFrame::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QFrame::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void ExprCSwatchFrame::selValChangedSignal(KSeExpr::Vec3d _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void ExprCSwatchFrame::swatchChanged(QColor _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN14ExprColorCurveE_t {};
} // unnamed namespace

template <> constexpr inline auto ExprColorCurve::qt_create_metaobjectdata<qt_meta_tag_ZN14ExprColorCurveE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ExprColorCurve",
        "selPosChangedSignal",
        "",
        "pos",
        "selValChangedSignal",
        "KSeExpr::Vec3d",
        "val",
        "swatchChanged",
        "QColor",
        "color",
        "cvSelectedSlot",
        "T_INTERP",
        "interp",
        "selPosChanged",
        "openDetail",
        "internalSwatchChanged"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'selPosChangedSignal'
        QtMocHelpers::SignalData<void(double)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Signal 'selValChangedSignal'
        QtMocHelpers::SignalData<void(KSeExpr::Vec3d)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'swatchChanged'
        QtMocHelpers::SignalData<void(QColor)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'cvSelectedSlot'
        QtMocHelpers::SlotData<void(double, KSeExpr::Vec3d, T_INTERP)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 }, { 0x80000000 | 5, 6 }, { 0x80000000 | 11, 12 },
        }}),
        // Slot 'selPosChanged'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'openDetail'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'internalSwatchChanged'
        QtMocHelpers::SlotData<void(QColor)>(15, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ExprColorCurve, qt_meta_tag_ZN14ExprColorCurveE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ExprColorCurve::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14ExprColorCurveE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14ExprColorCurveE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14ExprColorCurveE_t>.metaTypes,
    nullptr
} };

void ExprColorCurve::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ExprColorCurve *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->selPosChangedSignal((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 1: _t->selValChangedSignal((*reinterpret_cast<std::add_pointer_t<KSeExpr::Vec3d>>(_a[1]))); break;
        case 2: _t->swatchChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 3: _t->cvSelectedSlot((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KSeExpr::Vec3d>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<T_INTERP>>(_a[3]))); break;
        case 4: _t->selPosChanged(); break;
        case 5: _t->openDetail(); break;
        case 6: _t->internalSwatchChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ExprColorCurve::*)(double )>(_a, &ExprColorCurve::selPosChangedSignal, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ExprColorCurve::*)(KSeExpr::Vec3d )>(_a, &ExprColorCurve::selValChangedSignal, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ExprColorCurve::*)(QColor )>(_a, &ExprColorCurve::swatchChanged, 2))
            return;
    }
}

const QMetaObject *ExprColorCurve::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ExprColorCurve::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14ExprColorCurveE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int ExprColorCurve::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void ExprColorCurve::selPosChangedSignal(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void ExprColorCurve::selValChangedSignal(KSeExpr::Vec3d _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void ExprColorCurve::swatchChanged(QColor _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}
QT_WARNING_POP
