/****************************************************************************
** Meta object code from reading C++ file 'ExprColorSwatch.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.10.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../kseexpr/src/KSeExprUI/ExprColorSwatch.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ExprColorSwatch.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.10.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14ExprColorFrameE_t {};
} // unnamed namespace

template <> constexpr inline auto ExprColorFrame::qt_create_metaobjectdata<qt_meta_tag_ZN14ExprColorFrameE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ExprColorFrame",
        "selValChangedSignal",
        "",
        "KSeExpr::Vec3d",
        "value",
        "swatchChanged",
        "QColor",
        "color",
        "deleteSwatch",
        "ExprColorFrame*",
        "swatch",
        "deleteSwatchMenu",
        "QPoint",
        "pos"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'selValChangedSignal'
        QtMocHelpers::SignalData<void(KSeExpr::Vec3d)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'swatchChanged'
        QtMocHelpers::SignalData<void(QColor)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'deleteSwatch'
        QtMocHelpers::SignalData<void(ExprColorFrame *)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Slot 'deleteSwatchMenu'
        QtMocHelpers::SlotData<void(const QPoint &)>(11, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ExprColorFrame, qt_meta_tag_ZN14ExprColorFrameE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ExprColorFrame::staticMetaObject = { {
    QMetaObject::SuperData::link<QFrame::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14ExprColorFrameE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14ExprColorFrameE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14ExprColorFrameE_t>.metaTypes,
    nullptr
} };

void ExprColorFrame::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ExprColorFrame *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->selValChangedSignal((*reinterpret_cast<std::add_pointer_t<KSeExpr::Vec3d>>(_a[1]))); break;
        case 1: _t->swatchChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 2: _t->deleteSwatch((*reinterpret_cast<std::add_pointer_t<ExprColorFrame*>>(_a[1]))); break;
        case 3: _t->deleteSwatchMenu((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< ExprColorFrame* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ExprColorFrame::*)(KSeExpr::Vec3d )>(_a, &ExprColorFrame::selValChangedSignal, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ExprColorFrame::*)(QColor )>(_a, &ExprColorFrame::swatchChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ExprColorFrame::*)(ExprColorFrame * )>(_a, &ExprColorFrame::deleteSwatch, 2))
            return;
    }
}

const QMetaObject *ExprColorFrame::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ExprColorFrame::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14ExprColorFrameE_t>.strings))
        return static_cast<void*>(this);
    return QFrame::qt_metacast(_clname);
}

int ExprColorFrame::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QFrame::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void ExprColorFrame::selValChangedSignal(KSeExpr::Vec3d _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void ExprColorFrame::swatchChanged(QColor _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void ExprColorFrame::deleteSwatch(ExprColorFrame * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN15ExprColorWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto ExprColorWidget::qt_create_metaobjectdata<qt_meta_tag_ZN15ExprColorWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ExprColorWidget"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ExprColorWidget, qt_meta_tag_ZN15ExprColorWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ExprColorWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15ExprColorWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15ExprColorWidgetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15ExprColorWidgetE_t>.metaTypes,
    nullptr
} };

void ExprColorWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ExprColorWidget *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *ExprColorWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ExprColorWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15ExprColorWidgetE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int ExprColorWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN21ExprColorSwatchWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto ExprColorSwatchWidget::qt_create_metaobjectdata<qt_meta_tag_ZN21ExprColorSwatchWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ExprColorSwatchWidget",
        "selValChangedSignal",
        "",
        "KSeExpr::Vec3d",
        "val",
        "swatchChanged",
        "index",
        "swatchAdded",
        "swatchRemoved",
        "addNewColor",
        "removeSwatch",
        "ExprColorFrame*",
        "internalSwatchChanged",
        "QColor",
        "color"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'selValChangedSignal'
        QtMocHelpers::SignalData<void(KSeExpr::Vec3d)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'swatchChanged'
        QtMocHelpers::SignalData<void(int, KSeExpr::Vec3d)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 6 }, { 0x80000000 | 3, 4 },
        }}),
        // Signal 'swatchAdded'
        QtMocHelpers::SignalData<void(int, KSeExpr::Vec3d)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 6 }, { 0x80000000 | 3, 4 },
        }}),
        // Signal 'swatchRemoved'
        QtMocHelpers::SignalData<void(int)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 6 },
        }}),
        // Slot 'addNewColor'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'removeSwatch'
        QtMocHelpers::SlotData<void(ExprColorFrame *)>(10, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 11, 2 },
        }}),
        // Slot 'internalSwatchChanged'
        QtMocHelpers::SlotData<void(QColor)>(12, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ExprColorSwatchWidget, qt_meta_tag_ZN21ExprColorSwatchWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ExprColorSwatchWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21ExprColorSwatchWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21ExprColorSwatchWidgetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN21ExprColorSwatchWidgetE_t>.metaTypes,
    nullptr
} };

void ExprColorSwatchWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ExprColorSwatchWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->selValChangedSignal((*reinterpret_cast<std::add_pointer_t<KSeExpr::Vec3d>>(_a[1]))); break;
        case 1: _t->swatchChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KSeExpr::Vec3d>>(_a[2]))); break;
        case 2: _t->swatchAdded((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KSeExpr::Vec3d>>(_a[2]))); break;
        case 3: _t->swatchRemoved((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->addNewColor(); break;
        case 5: _t->removeSwatch((*reinterpret_cast<std::add_pointer_t<ExprColorFrame*>>(_a[1]))); break;
        case 6: _t->internalSwatchChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< ExprColorFrame* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ExprColorSwatchWidget::*)(KSeExpr::Vec3d )>(_a, &ExprColorSwatchWidget::selValChangedSignal, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ExprColorSwatchWidget::*)(int , KSeExpr::Vec3d )>(_a, &ExprColorSwatchWidget::swatchChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ExprColorSwatchWidget::*)(int , KSeExpr::Vec3d )>(_a, &ExprColorSwatchWidget::swatchAdded, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ExprColorSwatchWidget::*)(int )>(_a, &ExprColorSwatchWidget::swatchRemoved, 3))
            return;
    }
}

const QMetaObject *ExprColorSwatchWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ExprColorSwatchWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21ExprColorSwatchWidgetE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int ExprColorSwatchWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void ExprColorSwatchWidget::selValChangedSignal(KSeExpr::Vec3d _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void ExprColorSwatchWidget::swatchChanged(int _t1, KSeExpr::Vec3d _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void ExprColorSwatchWidget::swatchAdded(int _t1, KSeExpr::Vec3d _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void ExprColorSwatchWidget::swatchRemoved(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}
QT_WARNING_POP
